@@ -1,6 +1,7 @@
-from typing import Any, Dict, List, Optional, Union
+from typing import Any, Dict, List, Optional, Tuple, Union
 
 import cryptography
+import pyhpke
 from cryptography.hazmat.primitives import hashes
 from cryptography.hazmat.primitives.asymmetric import ec
 from cryptography.hazmat.primitives.asymmetric.ec import (
@@ -16,13 +17,15 @@ from cryptography.hazmat.primitives.kdf.hkdf import HKDF
 from ..const import (
     COSE_ALGORITHMS_CKDM_KEY_AGREEMENT,
     COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_ES,
+    COSE_ALGORITHMS_HPKE,
     COSE_ALGORITHMS_SIG_EC2,
     COSE_KEY_LEN,
     COSE_KEY_OPERATION_VALUES,
     COSE_KEY_TYPES,
 )
 from ..cose_key_interface import COSEKeyInterface
-from ..exceptions import EncodeError, VerifyError
+from ..exceptions import DecodeError, EncodeError, VerifyError
+from ..hpke_cipher_suite import HPKECipherSuite
 from ..utils import i2osp, os2ip, to_cis
 from .asymmetric import AsymmetricKey
 from .symmetric import AESCCMKey, AESGCMKey, ChaCha20Key, HMACKey
@@ -120,6 +123,12 @@ class EC2Key(AsymmetricKey):
                     if -2 not in params and -3 not in params:
                         # private key for key derivation.
                         self._key_ops = [7, 8]
+            elif self._alg in COSE_ALGORITHMS_HPKE.values():
+                if self._key_ops:
+                    if not (set(self._key_ops) & set([7, 8])):
+                        raise ValueError("Invalid key_ops for key derivation.")
+                else:
+                    self._key_ops = [7, 8]
             else:
                 raise ValueError(f"Unsupported or unknown alg(3) for EC2: {self._alg}.")
         else:
@@ -142,7 +151,7 @@ class EC2Key(AsymmetricKey):
                 if 2 in self._key_ops:
                     if len(self._key_ops) > 1:
                         raise ValueError("Invalid key_ops for public key.")
-                else:
+                elif not (set(self._key_ops) & set([7, 8])):
                     raise ValueError("Invalid key_ops for public key.")
 
         if self._alg in COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_ES.values():
@@ -266,6 +275,28 @@ class EC2Key(AsymmetricKey):
         except ValueError as err:
             raise VerifyError("Invalid signature.") from err
 
+    def seal(self, suite: HPKECipherSuite, msg: bytes, aad: bytes = b"") -> Tuple[bytes, bytes]:
+        # if self._alg != -1:
+        #     raise ValueError("alg should be HPKE(-1).");
+        if self._private_key is not None:
+            raise ValueError("Private key cannot be used for HPKE encryption.")
+
+        ctx = self._create_hpke_context(suite)
+        enc, sender = ctx.setup_send(self._public_key, b"")  # TODO: Add support for info
+        return enc, sender.aead.seal(aad, msg)
+
+    def open(self, suite: HPKECipherSuite, enc: bytes, msg: bytes, aad: bytes = b"") -> bytes:
+        # if self._alg != -1:
+        #     raise ValueError("alg should be HPKE(-1).");
+        if self._private_key is None:
+            raise ValueError("Public key cannot be used for HPKE decryption.")
+        ctx = self._create_hpke_context(suite)
+        try:
+            recipient = ctx.setup_recv(enc, self._private_key, b"")  # TODO: Add support for info
+            return recipient.aead.open(aad, msg)
+        except Exception as err:
+            raise DecodeError("Failed to decrypt.") from err
+
     def derive_key(
         self,
         context: Union[List[Any], Dict[str, Any]],
@@ -324,3 +355,26 @@ class EC2Key(AsymmetricKey):
         r = os2ip(sig[:num_bytes])
         s = os2ip(sig[num_bytes:])
         return encode_dss_signature(r, s)
+
+    def _create_hpke_context(self, suite: HPKECipherSuite) -> Any:
+        if self._crv == 1:
+            if suite.kem != 0x0010:
+                raise ValueError("KEM id should be 0x0010.")
+            if suite.kdf == 0x0001 and suite.aead == 0x0001:
+                return pyhpke.Suite__DHKEM_P256_HKDF_SHA256__HKDF_SHA256__AES_128_GCM
+            if suite.kdf == 0x0001 and suite.aead == 0x0003:
+                return pyhpke.Suite__DHKEM_P256_HKDF_SHA256__HKDF_SHA256__ChaCha20Poly1305
+            if suite.kdf == 0x0003 and suite.aead == 0x0001:
+                return pyhpke.Suite__DHKEM_P256_HKDF_SHA256__HKDF_SHA512__AES_128_GCM
+            raise ValueError("Unsupported kdf/aead combination.")
+        elif self._crv == 2:
+            if suite.kem != 0x0011:
+                raise ValueError("KEM id should be 0x0011.")
+            raise ValueError("Unsupported kdf/aead combination.")
+        elif self._crv == 3:
+            if suite.kem != 0x0012:
+                raise ValueError("KEM id should be 0x0012.")
+            if suite.kdf == 0x0001 and suite.aead == 0x0001:
+                return pyhpke.Suite__DHKEM_P521_HKDF_SHA512__HKDF_SHA512__AES_256_GCM
+            raise ValueError("Unsupported kdf/aead combination.")
+        raise ValueError("Invalid crv for HPKE.")
