@@ -6,6 +6,7 @@ from cbor2 import CBORTag
 from .cbor_processor import CBORProcessor
 from .const import COSE_ALGORITHMS_RECIPIENT
 from .cose_key_interface import COSEKeyInterface
+from .recipient_algs.hpke import HPKE
 from .recipient_interface import RecipientInterface
 from .recipients import Recipients
 from .signer import Signer
@@ -111,157 +112,11 @@ class COSE(CBORProcessor):
         self._verify_kid = verify_kid
         return
 
-    def encode_and_mac(
-        self,
-        payload: bytes,
-        key: COSEKeyInterface,
-        protected: Optional[Union[dict, bytes]] = None,
-        unprotected: Optional[dict] = None,
-        recipients: Optional[List[RecipientInterface]] = None,
-        external_aad: bytes = b"",
-        out: str = "",
-    ) -> Union[bytes, CBORTag]:
-        """
-        Encodes data with MAC.
-
-        Args:
-            payload (bytes): A content to be MACed.
-            key (COSEKeyInterface): A COSE key as a MAC Authentication key.
-            protected (Optional[Union[dict, bytes]]): Parameters that are to be cryptographically
-                protected.
-            unprotected (Optional[dict]): Parameters that are not cryptographically protected.
-            recipients (Optional[List[RecipientInterface]]): A list of recipient information structures.
-            external_aad(bytes): External additional authenticated data supplied by application.
-            out(str): An output format. Only ``"cbor2/CBORTag"`` can be used. If ``"cbor2/CBORTag"``
-                is specified. This function will return encoded data as
-                `cbor2 <https://cbor2.readthedocs.io/en/stable/>`_'s ``CBORTag`` object.
-                If any other value is specified, it will return encoded data as bytes.
-        Returns:
-            Union[bytes, CBORTag]: A byte string of the encoded COSE or a cbor2.CBORTag object.
-        Raises:
-            ValueError: Invalid arguments.
-            EncodeError: Failed to encode data.
-        """
-        p: Union[Dict[int, Any], bytes] = to_cose_header(protected) if not isinstance(protected, bytes) else protected
-        u = to_cose_header(unprotected)
-
-        ctx = "MAC0" if not recipients else "MAC"
-        b_protected = b""
-
-        # MAC0
-        if not recipients:
-            if isinstance(p, bytes):
-                b_protected = p
-            else:
-                if self._alg_auto_inclusion:
-                    p[1] = key.alg
-                b_protected = self._dumps(p)
-            if self._kid_auto_inclusion and key.kid:
-                u[4] = key.kid
-            mac_structure = [ctx, b_protected, external_aad, payload]
-            tag = key.sign(self._dumps(mac_structure))
-            res = CBORTag(17, [b_protected, u, payload, tag])
-            return res if out == "cbor2/CBORTag" else self._dumps(res)
-
-        # MAC
-        recs = []
-        for rec in recipients:
-            recs.append(rec.to_list())
-        if recipients[0].alg in COSE_ALGORITHMS_RECIPIENT.values():
-            if not isinstance(p, bytes):
-                if self._alg_auto_inclusion:
-                    p[1] = key.alg
-                if self._kid_auto_inclusion and key.kid:
-                    u[4] = key.kid
-        else:
-            raise NotImplementedError("Algorithms other than direct are not supported for recipients.")
-
-        if isinstance(p, bytes):
-            b_protected = p
-        else:
-            b_protected = self._dumps(p) if p else b""
-        mac_structure = [ctx, b_protected, external_aad, payload]
-        tag = key.sign(self._dumps(mac_structure))
-        cose_mac: List[Any] = [b_protected, u, payload, tag]
-        cose_mac.append(recs)
-        res = CBORTag(97, cose_mac)
-        return res if out == "cbor2/CBORTag" else self._dumps(res)
-
-    def encode_and_sign(
-        self,
-        payload: bytes,
-        key: Optional[COSEKeyInterface] = None,
-        protected: Optional[Union[dict, bytes]] = None,
-        unprotected: Optional[dict] = None,
-        signers: List[Signer] = [],
-        external_aad: bytes = b"",
-        out: str = "",
-    ) -> Union[bytes, CBORTag]:
-        """
-        Encodes data with signing.
-
-        Args:
-            payload (bytes): A content to be signed.
-            key (Optional[COSEKeyInterface]): A signing key for single signer
-                cases. When the ``signers`` parameter is set, this ``key`` will
-                be ignored and should not be set.
-            protected (Optional[Union[dict, bytes]]): Parameters that are to be
-                cryptographically protected.
-            unprotected (Optional[dict]): Parameters that are not cryptographically
-                protected.
-            signers (List[Signer]): A list of signer information objects for
-                multiple signer cases.
-            external_aad(bytes): External additional authenticated data supplied
-                by application.
-            out(str): An output format. Only ``"cbor2/CBORTag"`` can be used. If
-                ``"cbor2/CBORTag"`` is specified. This function will return encoded
-                data as `cbor2 <https://cbor2.readthedocs.io/en/stable/>`_'s
-                ``CBORTag`` object. If any other value is specified, it will return
-                encoded data as bytes.
-        Returns:
-            Union[bytes, CBORTag]: A byte string of the encoded COSE or a
-                cbor2.CBORTag object.
-        Raises:
-            ValueError: Invalid arguments.
-            EncodeError: Failed to encode data.
-        """
-        p: Union[Dict[int, Any], bytes] = to_cose_header(protected) if not isinstance(protected, bytes) else protected
-        u = to_cose_header(unprotected)
-
-        ctx = "Signature" if signers else "Signature1"
-        if not signers and key is not None:
-            if isinstance(p, dict) and self._alg_auto_inclusion:
-                p[1] = key.alg
-            if self._kid_auto_inclusion and key.kid:
-                u[4] = key.kid
-
-        b_protected = b""
-        if isinstance(p, bytes):
-            b_protected = p
-        else:
-            b_protected = self._dumps(p) if p else b""
-
-        # Signature1
-        if not signers and key is not None:
-            sig_structure = [ctx, b_protected, external_aad, payload]
-            sig = key.sign(self._dumps(sig_structure))
-            res = CBORTag(18, [b_protected, u, payload, sig])
-            return res if out == "cbor2/CBORTag" else self._dumps(res)
-
-        # Signature
-        sigs = []
-        for s in signers:
-            sig_structure = [ctx, b_protected, s.protected, external_aad, payload]
-            s.sign(self._dumps(sig_structure))
-            sigs.append([s.protected, s.unprotected, s.signature])
-        res = CBORTag(98, [b_protected, u, payload, sigs])
-        return res if out == "cbor2/CBORTag" else self._dumps(res)
-
     def encode_and_encrypt(
         self,
         payload: bytes,
-        key: COSEKeyInterface,
-        protected: Optional[Union[dict, bytes]] = None,
+        key: Optional[COSEKeyInterface] = None,
+        protected: Optional[dict] = None,
         unprotected: Optional[dict] = None,
         nonce: bytes = b"",
         recipients: Optional[List[RecipientInterface]] = None,
@@ -273,11 +128,9 @@ class COSE(CBORProcessor):
 
         Args:
             payload (bytes): A content to be encrypted.
-            key (COSEKeyInterface): A COSE key as an encryption key.
-            protected (Optional[Union[dict, bytes]]): Parameters that are to be
-                cryptographically protected.
-            unprotected (Optional[dict]): Parameters that are not cryptographically
-                protected.
+            key (Optional[COSEKeyInterface]): A content encryption key as COSEKey.
+            protected (Optional[dict]): Parameters that are to be cryptographically protected.
+            unprotected (Optional[dict]): Parameters that are not cryptographically protected.
             nonce (bytes): A nonce for encryption.
             recipients (Optional[List[RecipientInterface]]): A list of recipient
                 information structures.
@@ -295,58 +148,186 @@ class COSE(CBORProcessor):
             ValueError: Invalid arguments.
             EncodeError: Failed to encode data.
         """
-        p: Union[Dict[int, Any], bytes] = to_cose_header(protected) if not isinstance(protected, bytes) else protected
+        p = to_cose_header(protected)
         u = to_cose_header(unprotected)
-
-        ctx = "Encrypt0" if not recipients else "Encrypt"
-
-        if not nonce:
-            try:
-                nonce = key.generate_nonce()
-            except NotImplementedError:
-                raise ValueError("Nonce generation is not supported for the key. Set a nonce explicitly.")
+        if key is not None:
+            if self._alg_auto_inclusion:
+                p[1] = key.alg
+            if self._kid_auto_inclusion and key.kid:
+                u[4] = key.kid
+            if 1 not in p or p[1] != -1:  # not HPKE
+                if not nonce:
+                    try:
+                        nonce = key.generate_nonce()
+                    except NotImplementedError:
+                        raise ValueError("Nonce generation is not supported for the key. Set a nonce explicitly.")
+                u[5] = nonce
+        b_protected = self._dumps(p) if p else b""
+        ciphertext: bytes = b""
 
         # Encrypt0
         if not recipients:
-            if isinstance(p, bytes):
-                b_protected = p
-            else:
-                if self._alg_auto_inclusion:
-                    p[1] = key.alg
-                b_protected = self._dumps(p) if p else b""
-            if self._kid_auto_inclusion and key.kid:
-                u[4] = key.kid
-            u[5] = nonce
-            enc_structure = [ctx, b_protected, external_aad]
+            if 1 in p and p[1] == -1:  # HPKE
+                hpke = HPKE(p, u)
+                hpke.apply(recipient_key=key)
+                res = CBORTag(16, hpke.to_list(payload, external_aad, "Encrypt0"))
+                return res if out == "cbor2/CBORTag" else self._dumps(res)
+            if key is None:
+                raise ValueError("key should be set.")
+            enc_structure = ["Encrypt0", b_protected, external_aad]
             aad = self._dumps(enc_structure)
             ciphertext = key.encrypt(payload, nonce, aad)
             res = CBORTag(16, [b_protected, u, ciphertext])
             return res if out == "cbor2/CBORTag" else self._dumps(res)
 
         # Encrypt
-        recs = []
-        for rec in recipients:
-            recs.append(rec.to_list())
-        if recipients[0].alg in COSE_ALGORITHMS_RECIPIENT.values():
-            if not isinstance(p, bytes) and self._alg_auto_inclusion:
-                p[1] = key.alg
-            if self._kid_auto_inclusion and key.kid:
-                u[4] = key.kid
-            u[5] = nonce
-        else:
+        if recipients[0].alg not in COSE_ALGORITHMS_RECIPIENT.values():
             raise NotImplementedError("Algorithms other than direct are not supported for recipients.")
-        if isinstance(p, bytes):
-            b_protected = p
-        else:
-            b_protected = self._dumps(p) if p else b""
-        enc_structure = [ctx, b_protected, external_aad]
-        aad = self._dumps(enc_structure)
-        ciphertext = key.encrypt(payload, nonce, aad)
+
+        recs = []
+        is_hpke = True
+        for rec in recipients:
+            if rec.alg != -1:
+                is_hpke = False
+            recs.append(rec.to_list(payload, external_aad))
+
+        if 1 in p and p[1] == -1:  # HPKE
+            raise ValueError("alg for the first layer should not be HPKE.")
+
+        if key is not None:
+            enc_structure = ["Encrypt", b_protected, external_aad]
+            aad = self._dumps(enc_structure)
+            ciphertext = key.encrypt(payload, nonce, aad)
+        elif is_hpke is False:
+            raise ValueError("key should be set.")
+
         cose_enc: List[Any] = [b_protected, u, ciphertext]
         cose_enc.append(recs)
         res = CBORTag(96, cose_enc)
         return res if out == "cbor2/CBORTag" else self._dumps(res)
 
+    def encode_and_mac(
+        self,
+        payload: bytes,
+        key: COSEKeyInterface,
+        protected: Optional[dict] = None,
+        unprotected: Optional[dict] = None,
+        recipients: Optional[List[RecipientInterface]] = None,
+        external_aad: bytes = b"",
+        out: str = "",
+    ) -> Union[bytes, CBORTag]:
+        """
+        Encodes data with MAC.
+
+        Args:
+            payload (bytes): A content to be MACed.
+            key (COSEKeyInterface): A COSE key as a MAC Authentication key.
+            protected (Optional[dict]): Parameters that are to be cryptographically protected.
+            unprotected (Optional[dict]): Parameters that are not cryptographically protected.
+            recipients (Optional[List[RecipientInterface]]): A list of recipient information structures.
+            external_aad(bytes): External additional authenticated data supplied by application.
+            out(str): An output format. Only ``"cbor2/CBORTag"`` can be used. If ``"cbor2/CBORTag"``
+                is specified. This function will return encoded data as
+                `cbor2 <https://cbor2.readthedocs.io/en/stable/>`_'s ``CBORTag`` object.
+                If any other value is specified, it will return encoded data as bytes.
+        Returns:
+            Union[bytes, CBORTag]: A byte string of the encoded COSE or a cbor2.CBORTag object.
+        Raises:
+            ValueError: Invalid arguments.
+            EncodeError: Failed to encode data.
+        """
+        p = to_cose_header(protected)
+        u = to_cose_header(unprotected)
+        if self._alg_auto_inclusion:
+            p[1] = key.alg
+        if self._kid_auto_inclusion and key.kid:
+            u[4] = key.kid
+        b_protected = self._dumps(p) if p else b""
+
+        # MAC0
+        if not recipients:
+            mac_structure = ["MAC0", b_protected, external_aad, payload]
+            tag = key.sign(self._dumps(mac_structure))
+            res = CBORTag(17, [b_protected, u, payload, tag])
+            return res if out == "cbor2/CBORTag" else self._dumps(res)
+
+        # MAC
+        if recipients[0].alg not in COSE_ALGORITHMS_RECIPIENT.values():
+            raise NotImplementedError("Algorithms other than direct are not supported for recipients.")
+
+        mac_structure = ["MAC", b_protected, external_aad, payload]
+        tag = key.sign(self._dumps(mac_structure))
+
+        recs = []
+        for rec in recipients:
+            recs.append(rec.to_list())
+
+        cose_mac: List[Any] = [b_protected, u, payload, tag]
+        cose_mac.append(recs)
+        res = CBORTag(97, cose_mac)
+        return res if out == "cbor2/CBORTag" else self._dumps(res)
+
+    def encode_and_sign(
+        self,
+        payload: bytes,
+        key: Optional[COSEKeyInterface] = None,
+        protected: Optional[dict] = None,
+        unprotected: Optional[dict] = None,
+        signers: List[Signer] = [],
+        external_aad: bytes = b"",
+        out: str = "",
+    ) -> Union[bytes, CBORTag]:
+        """
+        Encodes data with signing.
+
+        Args:
+            payload (bytes): A content to be signed.
+            key (Optional[COSEKeyInterface]): A signing key for single signer
+                cases. When the ``signers`` parameter is set, this ``key`` will
+                be ignored and should not be set.
+            protected (Optional[dict]): Parameters that are to be cryptographically protected.
+            unprotected (Optional[dict]): Parameters that are not cryptographically protected.
+            signers (List[Signer]): A list of signer information objects for
+                multiple signer cases.
+            external_aad(bytes): External additional authenticated data supplied
+                by application.
+            out(str): An output format. Only ``"cbor2/CBORTag"`` can be used. If
+                ``"cbor2/CBORTag"`` is specified. This function will return encoded
+                data as `cbor2 <https://cbor2.readthedocs.io/en/stable/>`_'s
+                ``CBORTag`` object. If any other value is specified, it will return
+                encoded data as bytes.
+        Returns:
+            Union[bytes, CBORTag]: A byte string of the encoded COSE or a
+                cbor2.CBORTag object.
+        Raises:
+            ValueError: Invalid arguments.
+            EncodeError: Failed to encode data.
+        """
+        p = to_cose_header(protected)
+        u = to_cose_header(unprotected)
+        if key is not None:
+            if self._alg_auto_inclusion:
+                p[1] = key.alg
+            if self._kid_auto_inclusion and key.kid:
+                u[4] = key.kid
+        b_protected = self._dumps(p) if p else b""
+
+        # Signature1
+        if not signers and key is not None:
+            sig_structure = ["Signature1", b_protected, external_aad, payload]
+            sig = key.sign(self._dumps(sig_structure))
+            res = CBORTag(18, [b_protected, u, payload, sig])
+            return res if out == "cbor2/CBORTag" else self._dumps(res)
+
+        # Signature
+        sigs = []
+        for s in signers:
+            sig_structure = ["Signature", b_protected, s.protected, external_aad, payload]
+            s.sign(self._dumps(sig_structure))
+            sigs.append([s.protected, s.unprotected, s.signature])
+        res = CBORTag(98, [b_protected, u, payload, sigs])
+        return res if out == "cbor2/CBORTag" else self._dumps(res)
+
     def decode(
         self,
         data: Union[bytes, CBORTag],
@@ -411,7 +392,7 @@ class COSE(CBORProcessor):
         else:
             raise ValueError(f"Unsupported or unknown CBOR tag({data.tag}).")
 
-        protected = self._loads(data.value[0]) if data.value[0] else b""
+        protected: Union[Dict[int, Any], bytes] = self._loads(data.value[0]) if data.value[0] else b""
         unprotected = data.value[1]
         if not isinstance(unprotected, dict):
             raise ValueError("unprotected header should be dict.")
@@ -429,6 +410,9 @@ class COSE(CBORProcessor):
                     if k.kid != kid:
                         continue
                     try:
+                        if not isinstance(protected, bytes) and alg == -1:  # HPKE
+                            hpke = HPKE(protected, unprotected, data.value[2])
+                            return hpke.decrypt(k, external_aad=external_aad, aad_context="Encrypt0")
                         return k.decrypt(data.value[2], nonce, aad)
                     except Exception as e:
                         err = e
@@ -443,10 +427,9 @@ class COSE(CBORProcessor):
         # Encrypt
         if data.tag == 96:
             aad = self._dumps(["Encrypt", data.value[0], external_aad])
-            nonce = unprotected.get(5, None)
             rs = Recipients.from_list(data.value[3], self._verify_kid)
-            enc_key = rs.extract(keys, context, alg)
-            return enc_key.decrypt(data.value[2], nonce, aad)
+            nonce = unprotected.get(5, b"")
+            return rs.decrypt(keys, alg, context, data.value[2], nonce, aad, external_aad)
 
         # MAC0
         if data.tag == 17:
