@@ -32,7 +32,7 @@ class Recipients:
         alg: int = 0,
     ) -> COSEKeyInterface:
         """
-        Decodes an appropriate key from recipients or keys privided as a parameter ``keys``.
+        Decodes an appropriate key from recipients or keys provided as a parameter ``keys``.
         """
         if not self._recipients:
             raise ValueError("No recipients.")
@@ -45,13 +45,49 @@ class Recipients:
                     if k.kid != r.kid:
                         continue
                     try:
-                        return r.extract(k, alg=alg, context=context)
+                        return r.extract(k, alg, context)
                     except Exception as e:
                         err = e
                 continue
             for k in keys:
                 try:
-                    return r.extract(k, alg=alg, context=context)
+                    return r.extract(k, alg, context)
+                except Exception as e:
+                    err = e
+        raise err
+
+    def decrypt(
+        self,
+        keys: List[COSEKeyInterface],
+        alg: int = 0,
+        context: Optional[Union[Dict[str, Any], List[Any]]] = None,
+        payload: bytes = b"",
+        nonce: bytes = b"",
+        aad: bytes = b"",
+        external_aad: bytes = b"",
+    ) -> bytes:
+
+        """
+        Decrypts the supplied payload.
+        """
+        if not self._recipients:
+            raise ValueError("No recipients.")
+        err: Exception = ValueError("key is not found.")
+        for r in self._recipients:
+            if not r.kid and self._verify_kid:
+                raise ValueError("kid should be specified in recipient.")
+            if r.kid:
+                for k in keys:
+                    if k.kid != r.kid:
+                        continue
+                    try:
+                        return r.decrypt(k, alg, context, payload, nonce, aad, external_aad)
+                    except Exception as e:
+                        err = e
+                continue
+            for k in keys:
+                try:
+                    return r.decrypt(k, alg, context, payload, nonce, aad, external_aad)
                 except Exception as e:
                     err = e
         raise err
