@@ -7,6 +7,7 @@ import pytest
 from cwt.algs.ec2 import EC2Key
 from cwt.cose_key import COSEKey
 from cwt.exceptions import VerifyError
+from cwt.hpke_cipher_suite import HPKECipherSuite
 
 from .utils import key_path
 
@@ -585,7 +586,8 @@ class TestEC2Key:
                     -1: 1,
                     4: [7, 8],
                 },
-                "Invalid key_ops for public key.",
+                "x(-2) not found.",
+                # "Invalid key_ops for public key.",
             ),
             (
                 {
@@ -809,3 +811,379 @@ class TestEC2Key:
         with pytest.raises(ValueError) as err:
             EC2Key.to_cose_key(private_key.key)
         assert "Unsupported or unknown key for EC2." in str(err.value)
+
+    @pytest.mark.parametrize(
+        "kdf, aead",
+        [
+            (0x0001, 0x0001),
+            # (0x0001, 0x0002),
+            (0x0001, 0x0003),
+            # (0x0002, 0x0001),
+            # (0x0002, 0x0002),
+            # (0x0002, 0x0003),
+            (0x0003, 0x0001),
+            # (0x0003, 0x0002),
+            # (0x0003, 0x0003),
+        ],
+    )
+    def test_ec2_key_hpke_ec_p256(self, kdf, aead):
+        suite = HPKECipherSuite(kem=0x0010, kdf=kdf, aead=aead)
+        priv_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+                "d": "V8kgd2ZBRuh2dgyVINBUqpPDr7BOMGcF22CQMIUHtNM",
+            }
+        )
+        pub_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+            }
+        )
+        enc, ciphertext = pub_key.seal(suite, b"Hello world!")
+        assert b"Hello world!" == priv_key.open(suite, enc, ciphertext)
+        with pytest.raises(ValueError) as err:
+            pub_key.open(suite, b"", b"")
+            pytest.fail("Public key cannot be used for HPKE decryption.")
+        assert "." in str(err.value)
+        with pytest.raises(ValueError) as err:
+            priv_key.seal(suite, b"")
+            pytest.fail("Private key cannot be used for HPKE encryption.")
+        assert "." in str(err.value)
+
+    @pytest.mark.parametrize(
+        "kdf, aead",
+        [
+            # (0x0001, 0x0001),
+            (0x0001, 0x0002),
+            # (0x0001, 0x0003),
+            (0x0002, 0x0001),
+            (0x0002, 0x0002),
+            (0x0002, 0x0003),
+            # (0x0003, 0x0001),
+            (0x0003, 0x0002),
+            (0x0003, 0x0003),
+        ],
+    )
+    def test_ec2_key_hpke_ec_p256_unsupported(self, kdf, aead):
+        suite = HPKECipherSuite(kem=0x0010, kdf=kdf, aead=aead)
+        pub_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+            }
+        )
+        with pytest.raises(ValueError) as err:
+            pub_key.seal(suite, b"Hello world!")
+            pytest.fail("seal should fail.")
+        assert "Unsupported kdf/aead combination." in str(err.value)
+
+    @pytest.mark.parametrize(
+        "kem",
+        [
+            0x0011,
+            0x0012,
+            0x0020,
+            0x0021,
+        ],
+    )
+    def test_ec2_key_hpke_ec_p256_with_invalid_kem(self, kem):
+        suite = HPKECipherSuite(kem=kem, kdf=0x0001, aead=0x0001)
+        pub_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+            }
+        )
+        with pytest.raises(ValueError) as err:
+            pub_key.seal(suite, b"Hello world!")
+            pytest.fail("seal should fail.")
+        assert "KEM id should be 0x0010." in str(err.value)
+
+    @pytest.mark.parametrize(
+        "kdf, aead",
+        [
+            (0x0001, 0x0001),
+            (0x0001, 0x0002),
+            (0x0001, 0x0003),
+            (0x0002, 0x0001),
+            (0x0002, 0x0002),
+            (0x0002, 0x0003),
+            (0x0003, 0x0001),
+            (0x0003, 0x0002),
+            (0x0003, 0x0003),
+        ],
+    )
+    def test_ec2_key_hpke_ec_p384_unsupported(self, kdf, aead):
+        suite = HPKECipherSuite(kem=0x0011, kdf=kdf, aead=aead)
+        pub_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "crv": "P-384",
+                "kid": "P-384-01",
+                "x": "_XyN9woHaS0mPimSW-etwJMEDSzxIMjp4PjezavU8SHJoClz1bQrcmPb1ZJxHxhI",
+                "y": "GCNfc32p9sRotx7u2oDGJ3Eqz6q5zPHLdizNn83oRsUTN31eCWfGLHWRury3xF50",
+                # "d": "1pImEKbrr771-RKi8Tb7tou_WjiR7kwui_nMu16449rk3lzAqf9buUhTkJ-pogkb",
+            }
+        )
+        with pytest.raises(ValueError) as err:
+            pub_key.seal(suite, b"Hello world!")
+            pytest.fail("seal should fail.")
+        assert "Unsupported kdf/aead combination." in str(err.value)
+
+    @pytest.mark.parametrize(
+        "kem",
+        [
+            0x0010,
+            0x0012,
+            0x0020,
+            0x0021,
+        ],
+    )
+    def test_ec2_key_hpke_ec_p384_with_invalid_kem(self, kem):
+        suite = HPKECipherSuite(kem=kem, kdf=0x0001, aead=0x0001)
+        pub_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "crv": "P-384",
+                "kid": "P-384-01",
+                "x": "_XyN9woHaS0mPimSW-etwJMEDSzxIMjp4PjezavU8SHJoClz1bQrcmPb1ZJxHxhI",
+                "y": "GCNfc32p9sRotx7u2oDGJ3Eqz6q5zPHLdizNn83oRsUTN31eCWfGLHWRury3xF50",
+            }
+        )
+        with pytest.raises(ValueError) as err:
+            pub_key.seal(suite, b"Hello world!")
+            pytest.fail("seal should fail.")
+        assert "KEM id should be 0x0011." in str(err.value)
+
+    @pytest.mark.parametrize(
+        "kdf, aead",
+        [
+            (0x0001, 0x0001),
+            # (0x0001, 0x0002),
+            # (0x0001, 0x0003),
+            # (0x0002, 0x0001),
+            # (0x0002, 0x0002),
+            # (0x0002, 0x0003),
+            # (0x0003, 0x0001),
+            # (0x0003, 0x0002),
+            # (0x0003, 0x0003),
+        ],
+    )
+    def test_ec2_key_hpke_ec_p521(self, kdf, aead):
+        suite = HPKECipherSuite(kem=0x0012, kdf=kdf, aead=aead)
+        priv_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "crv": "P-521",
+                "kid": "P-521-01",
+                "x": "APkZitSJMJUMB-iPCt47sWu_CrnUHg6IAR4qjmHON-2u41Rjg6DNOS0LZYJJt-AVH5NgGVi8ElIfjo71b9HXCTOc",
+                "y": "ASx-Cb--149HJ-e1KlSaY-1BOhwOdcTkxSt8BGbW7_hnGfzHsoXM3ywwNcp1Yad-FHUKwmCyMelMQEn2Rh4V2l3I",
+                "d": "ADYyo73ZKicOjwGDYQ_ybZKnVzdAcxGm9OVAxQjzgVM4jaS-Iwtkz90oLdDz3shgKlDgtRK2Aa9lMhqR94hBo4IE",
+            }
+        )
+        pub_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "crv": "P-521",
+                "kid": "P-521-01",
+                "x": "APkZitSJMJUMB-iPCt47sWu_CrnUHg6IAR4qjmHON-2u41Rjg6DNOS0LZYJJt-AVH5NgGVi8ElIfjo71b9HXCTOc",
+                "y": "ASx-Cb--149HJ-e1KlSaY-1BOhwOdcTkxSt8BGbW7_hnGfzHsoXM3ywwNcp1Yad-FHUKwmCyMelMQEn2Rh4V2l3I",
+            }
+        )
+        enc, ciphertext = pub_key.seal(suite, b"Hello world!")
+        assert b"Hello world!" == priv_key.open(suite, enc, ciphertext)
+
+    @pytest.mark.parametrize(
+        "kdf, aead",
+        [
+            # (0x0001, 0x0001),
+            (0x0001, 0x0002),
+            (0x0001, 0x0003),
+            (0x0002, 0x0001),
+            (0x0002, 0x0002),
+            (0x0002, 0x0003),
+            (0x0003, 0x0001),
+            (0x0003, 0x0002),
+            (0x0003, 0x0003),
+        ],
+    )
+    def test_ec2_key_hpke_ec_p521_unsupported(self, kdf, aead):
+        suite = HPKECipherSuite(kem=0x0012, kdf=kdf, aead=aead)
+        pub_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "crv": "P-521",
+                "kid": "P-521-01",
+                "x": "APkZitSJMJUMB-iPCt47sWu_CrnUHg6IAR4qjmHON-2u41Rjg6DNOS0LZYJJt-AVH5NgGVi8ElIfjo71b9HXCTOc",
+                "y": "ASx-Cb--149HJ-e1KlSaY-1BOhwOdcTkxSt8BGbW7_hnGfzHsoXM3ywwNcp1Yad-FHUKwmCyMelMQEn2Rh4V2l3I",
+                # "d": "ADYyo73ZKicOjwGDYQ_ybZKnVzdAcxGm9OVAxQjzgVM4jaS-Iwtkz90oLdDz3shgKlDgtRK2Aa9lMhqR94hBo4IE",
+            }
+        )
+        with pytest.raises(ValueError) as err:
+            pub_key.seal(suite, b"Hello world!")
+            pytest.fail("seal should fail.")
+        assert "Unsupported kdf/aead combination." in str(err.value)
+
+    @pytest.mark.parametrize(
+        "kem",
+        [
+            0x0010,
+            0x0011,
+            0x0020,
+            0x0021,
+        ],
+    )
+    def test_ec2_key_hpke_ec_p521_with_invalid_kem(self, kem):
+        suite = HPKECipherSuite(kem=kem, kdf=0x0001, aead=0x0001)
+        pub_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "crv": "P-521",
+                "kid": "P-521-01",
+                "x": "APkZitSJMJUMB-iPCt47sWu_CrnUHg6IAR4qjmHON-2u41Rjg6DNOS0LZYJJt-AVH5NgGVi8ElIfjo71b9HXCTOc",
+                "y": "ASx-Cb--149HJ-e1KlSaY-1BOhwOdcTkxSt8BGbW7_hnGfzHsoXM3ywwNcp1Yad-FHUKwmCyMelMQEn2Rh4V2l3I",
+            }
+        )
+        with pytest.raises(ValueError) as err:
+            pub_key.seal(suite, b"Hello world!")
+            pytest.fail("seal should fail.")
+        assert "KEM id should be 0x0012." in str(err.value)
+
+    def test_ec2_key_hpke_ec_with_invalid_crv(self):
+        suite = HPKECipherSuite(kem=0x0010, kdf=0x0001, aead=0x0001)
+        pub_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "use": "sig",
+                "crv": "secp256k1",
+                "kid": "secp256k1-01",
+                "x": "6_W-2tVwr9SU0Ts9sFmZgt8JrlxDi8d6G5ltwL2futU",
+                "y": "mEOme4oiQsXnNwtlfyH5hEkvLDxrtqDUY0UXBIMRuL0",
+            }
+        )
+        with pytest.raises(ValueError) as err:
+            pub_key.seal(suite, b"Hello world!")
+            pytest.fail("seal should fail.")
+        assert "Invalid crv for HPKE." in str(err.value)
+
+    def test_ec2_key_hpke_with_alg_hpke(self):
+        suite = HPKECipherSuite(kem=0x0010, kdf=0x0001, aead=0x0001)
+        priv_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                "alg": "HPKE",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+                "d": "V8kgd2ZBRuh2dgyVINBUqpPDr7BOMGcF22CQMIUHtNM",
+            }
+        )
+        pub_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                "alg": "HPKE",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+            }
+        )
+        enc, ciphertext = pub_key.seal(suite, b"Hello world!")
+        plaintext = priv_key.open(suite, enc, ciphertext)
+        assert plaintext == b"Hello world!"
+
+    @pytest.mark.parametrize(
+        "key_ops",
+        [
+            ["sign"],
+            ["verify"],
+            ["sign", "verify"],
+        ],
+    )
+    def test_ec2_key_hpke_with_alg_hpke_and_invalid_key_ops(self, key_ops):
+        with pytest.raises(ValueError) as err:
+            COSEKey.from_jwk(
+                {
+                    "kty": "EC",
+                    "kid": "01",
+                    "crv": "P-256",
+                    "alg": "HPKE",
+                    "key_ops": key_ops,
+                    "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                    "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+                    "d": "V8kgd2ZBRuh2dgyVINBUqpPDr7BOMGcF22CQMIUHtNM",
+                }
+            )
+        assert "Invalid key_ops for key derivation." in str(err.value)
+
+    @pytest.mark.parametrize(
+        "key_ops",
+        [
+            ["sign"],
+        ],
+    )
+    def test_ec2_key_hpke_with_invalid_key_ops(self, key_ops):
+        with pytest.raises(ValueError) as err:
+            COSEKey.from_jwk(
+                {
+                    "kty": "EC",
+                    "kid": "01",
+                    "crv": "P-256",
+                    # "alg": "HPKE",
+                    "key_ops": key_ops,
+                    "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                    "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+                    # "d": "V8kgd2ZBRuh2dgyVINBUqpPDr7BOMGcF22CQMIUHtNM",
+                }
+            )
+        assert "Invalid key_ops for public key." in str(err.value)
+
+    @pytest.mark.parametrize(
+        "key_ops",
+        [
+            ["deriveBits"],
+            ["deriveKey"],
+            ["deriveBits", "deriveKey"],
+        ],
+    )
+    def test_ec2_key_hpke_with_key_ops(self, key_ops):
+        suite = HPKECipherSuite(kem=0x0010, kdf=0x0001, aead=0x0001)
+        priv_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                "key_ops": key_ops,
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+                "d": "V8kgd2ZBRuh2dgyVINBUqpPDr7BOMGcF22CQMIUHtNM",
+            }
+        )
+        pub_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                "key_ops": key_ops,
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+            }
+        )
+        enc, ciphertext = pub_key.seal(suite, b"Hello world!")
+        plaintext = priv_key.open(suite, enc, ciphertext)
+        assert plaintext == b"Hello world!"
